/*
 *				Twidere - Twitter client for Android
 * 
 * Copyright (C) 2012 Mariotaku Lee <mariotaku.lee@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.mariotaku.twidere.fragment;

import java.util.List;

import org.mariotaku.twidere.loader.ListSubscribersLoader;
import org.mariotaku.twidere.model.ParcelableUser;

import android.os.Bundle;
import android.support.v4.content.Loader;

public class UserListSubscribersFragment extends BaseUsersListFragment {

	private long mCursor = -1;

	@Override
	public Loader<List<ParcelableUser>> newLoaderInstance() {
		final Bundle args = getArguments();
		int list_id = -1;
		long account_id = -1, user_id = -1;
		String screen_name = null, list_name = null;
		if (args != null) {
			list_id = args.getInt(INTENT_KEY_LIST_ID, -1);
			account_id = args.getLong(INTENT_KEY_ACCOUNT_ID, -1);
			user_id = args.getLong(INTENT_KEY_USER_ID, -1);
			screen_name = args.getString(INTENT_KEY_SCREEN_NAME);
			list_name = args.getString(INTENT_KEY_LIST_NAME);
		}
		return new ListSubscribersLoader(getActivity(), account_id, list_id, user_id, screen_name, list_name, mCursor,
				getData());
	}

	@Override
	public void onActivityCreated(Bundle savedInstanceState) {
		if (savedInstanceState != null) {
			mCursor = savedInstanceState.getLong(INTENT_KEY_PAGE, -1);
		}
		super.onActivityCreated(savedInstanceState);
	}

	@Override
	public void onDestroyView() {
		mCursor = -1;
		super.onDestroyView();
	}

	@Override
	public void onLoadFinished(Loader<List<ParcelableUser>> loader, List<ParcelableUser> data) {
		if (loader instanceof ListSubscribersLoader) {
			final long cursor = ((ListSubscribersLoader) loader).getNextCursor();
			final boolean all_items_loaded = cursor != -2 && cursor == mCursor;
			getListAdapter().setShowLastItemAsGap(!all_items_loaded);
			setAllItemsLoaded(all_items_loaded);
			if (cursor != -2) {
				mCursor = cursor;
			}
		}
		super.onLoadFinished(loader, data);
	}

	@Override
	public void onSaveInstanceState(Bundle outState) {
		outState.putLong(INTENT_KEY_PAGE, mCursor);
		super.onSaveInstanceState(outState);
	}

}
